/**
 * AcknowledgePdu database operations (DML's)
 */
package pduDB;

import java.sql.*;
import java.util.*;


import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

import iwt.mine.*;

public class AcknowledgePduDML {

 private AcknowledgePdu aPdu;
 private SQLUtil db; //db connection
 private static String acknowledge_tablename = "dis_acknowledgepdu_table";
 private static String[] acknowledge_cols = { //make sure this is in the order
  					    // of the columns in the table
        "id", "disheader_id", "requestid","responseflag","acknowledgeflag"
        };
 private static String qMarks = "?,?,?,?";
		
 private String sql = "insert into " + acknowledge_tablename;
 private Hashtable ht;
 private int disHeaderId;

 public AcknowledgePduDML() {
     System.out.println("AcknowledgePdu called");
 }
 public AcknowledgePduDML(AcknowledgePdu pdu, int disId, SQLUtil dbUtil) {
     aPdu = pdu;
     db = dbUtil;
     disHeaderId = disId;
     fillHash();
 }
 public AcknowledgePduDML(int disId, SQLUtil dbUtil) {
     aPdu = new AcknowledgePdu();
     db = dbUtil;
     ht = null;
     disHeaderId = disId;
     fillPdu();
 }

 private void fillHash(){
     ht = new Hashtable();
     ht.put("disheader_id",(new Integer(disHeaderId)).toString());

     ht.put("requestid",aPdu.getRequestID().toString());
     ht.put("responseflag",aPdu.getResponseFlag().toString());
     ht.put("acknowledgeflag",aPdu.getAcknowledgeFlag().toString());
     
  //   System.out.println("HT in fill hash = "+ ht.toString());
 
 }

 private void fillPdu(){
     String sql = "select * from "+acknowledge_tablename +
 	          " where disheader_id = " +disHeaderId;
     int acknowledge_id = -1;
     try{
     ht = db.getRecord(sql);
     if(ht == null) {
         msg("Error !!! No acknowledge record found for this disheader ");
         return;
     }
     acknowledge_id = getInt("id");
   
     aPdu.setRequestID(getInt("requestid"));
     aPdu.setAcknowledgeFlag(getInt("acknowledgeflag"));
     aPdu.setResponseFlag(getInt("responseflag"));
     //now clear the hashtable ht for the new queries
     ht.clear();

     }catch(Exception e) {msg(e.getMessage());}     
 }

 private String get(String name) {
     return ht.get(name).toString();
 }

 private int getInt(String name) {
     int i = 0;
     try {
         i = Integer.parseInt(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return i;
 }

 public void save(){
     sql = makeSQLStr(sql);
     //msg("opening database connection after making sql str");
     //db.open();
    // msg("sql string end product " +sql);
     db.insert(ht,acknowledge_tablename,sql,1); 
     //msg("Inserted on PDU...");
 }

 private String makeSQLStr(String s) {
     StringBuffer sb = new StringBuffer(s + "(");
     for(int i = 1; i < acknowledge_cols.length; i++) {
         if(i < acknowledge_cols.length-1)
	         sb.append(acknowledge_cols[i].toLowerCase() +",");
         else
			 sb.append(acknowledge_cols[i].toLowerCase() + ")");
     }
     sb.append(" values ( " + qMarks + " ) ");     
     return sb.toString();
 }

 private void msg(String str) {
     System.out.println(str);
 }
 
 public AcknowledgePdu getPDU() {
     return aPdu;
 }
}
