/**
 * DISPduLoggerServlet.java
 * 
 * This servlets accepts DIS PDUs and loads it into the
 * database. 
 *
 */
package pduDB;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.net.*;
import java.sql.*;
import java.util.*;

import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

import org.w3c.dom.*;
import com.docuverse.freedom.*;

import iwt.domUtil.*;
import iwt.*;
import iwt.mine.*;

import comm.PDUReadWrite;
 
public class DISPduLoggerServlet extends HttpServlet {

 private static SQLUtil db; //db connection
 private String sql;
 private Hashtable ht;
 private String action = "";

 public static final boolean DEBUG = true;


 
/**
 * Handle the POST HTTP REQUEST Method
 * One line as name1=value1&name2=value2&...(URL encoded name-value pairs)
 * and this line should start with action parameter name..
 * and rest is assumed as a well formed XML document
 */

 public void doPost(HttpServletRequest req, HttpServletResponse res)
                    throws IOException {

     msg("Inside POST method ");
     ServletOutputStream out = res.getOutputStream();

     BufferedReader inpd = 
                new BufferedReader(new InputStreamReader(req.getInputStream()));
     String line;
     StringBuffer linebuf = new StringBuffer();
     while ((line = inpd.readLine()) != null) {
         if(!(line.startsWith("action"))){
             linebuf.ensureCapacity(line.length()*2);
             linebuf.append(line+"\n");
         }else
             getParams(line);  // one line with name=value attributes
     }
     //msg("##"+linebuf.toString()+"###");
     createPdus(linebuf.toString());
     //db.close();
     line = null;
     linebuf = null; 
     inpd.close();
     out.println("Received all the input");
     out.close();  
     System.gc();
 }

/**
 *  Creates the DOM object and creates the PDU's
 */
 private void createPdus(String buf) {
 
     try{
         Freedom dom = new Freedom("com.docuverse.freedom.SAXDriver");
         dom.setProperty("sax.driver","com.microstar.xml.SAXDriver");

         StringReader sr = new StringReader(buf);
         Document doc = dom.openDocument(sr);

         readXML(doc);

      } catch( Exception mue)
        { 
        msg(mue.getMessage());
        mue.printStackTrace(System.out);
        System.out.println(" File is not readable! "); }
 }

 private int readXML(Document doc) {
      PDUTypeField aTypeField = new PDUTypeField();
      ProtocolDataUnit aPdu;
      int[] frequency = new int[256];
      int i=0;
      int ret=0, iret=0;
      printDOM aPrinter = new printDOM();

      for(i=0;i<256;i++) frequency[i]=0;

  //    System.out.println(" In readXML ");
      //pdu.insert(doc);
      //pdu.write(doc);
      //aPrinter.print(doc,"",System.out);
//      System.out.println(" DOM is ready ");

      Element rNode = doc.getDocumentElement();

      if( rNode!=null )
      {
         NodeList aList = rNode.getElementsByTagName(DIS_TAGS.PDU);

         ret = aList.getLength();
//         System.out.println("Total # of PDUs = "+ret);

         i=0;
         while( i<ret )
         {
           Node aNode = aList.item(i);
           if( aNode.getNodeName().equals(DIS_TAGS.PDU) )
           {
            //aPdu = processANode((Element)aNode);
            aPdu = PDUReadWrite.getPDU((Element)aNode);
            if( aPdu!=null && aPdu.getPduType().shortValue()>-1 &&
                aPdu.getPduType().shortValue()<256 )
                { frequency[aPdu.getPduType().shortValue()]++; iret++; 
//            System.out.println(" PDU Type inside readXML ="+aPdu.getPduType().shortValue());
            	  if(action.equals("save")) 
				savePdu(aPdu);
                 }

          }// end of if

          i++;

        }// end of while

      }// end of if


      for(i=0;i<256;i++)
          if(frequency[i]>0)
             System.out.println(" F['"+aTypeField.toString(i)+"']: "+frequency[i ]);

      System.out.println(" There are "+iret+" pdus in the file! ");

      return ret;

}// end of readXML()


/**
 * Saves the PDU data into the database
 */
 private synchronized void savePdu(ProtocolDataUnit aPdu) { 
     DISHeaderDML disHeader; 
     int disHeaderId = -1;

    // if( aPdu!=null && aPdu.getPduType().shortValue()>-1 &&
      //          aPdu.getPduType().shortValue()<256 ){
         //msg("Before the switch statement");
         //System.out.println(" PDU Type inside savePDU ="+aPdu.getPduType().shortValue());
         disHeader = new DISHeaderDML(aPdu,db);
         disHeader.save();
         disHeaderId = disHeader.getId();
         //System.out.println(" PDU Type afterdisHeader Save ="+aPdu.getPduType().shortValue());
         switch(aPdu.getPduType().shortValue()) {
         
          // entity state pdu
         case PDUTypeField.ENTITYSTATE:
              EntityStatePduDML esPdu = new EntityStatePduDML((EntityStatePdu)aPdu, disHeaderId, db);
              esPdu.save();
              break;

         // Fire pdu
         case PDUTypeField.FIRE:
              FirePduDML fPdu = new FirePduDML((FirePdu)aPdu, disHeaderId, db);
              fPdu.save();
              break;

         // Detonation pdu
         case PDUTypeField.DETONATION:
              DetonationPduDML dPdu = new DetonationPduDML((DetonationPdu)aPdu, disHeaderId, db);
              dPdu.save();
              break;

         // Mine pdu
         case PDUTypeField.MINE:
              MinePduDML mPdu = new MinePduDML((MinePdu)aPdu, disHeaderId, db);
              mPdu.save();
              break;

         // Minefield pdu
         case PDUTypeField.MINEFIELD:
              MinefieldPduDML mfPdu = new MinefieldPduDML((MinefieldPdu)aPdu, disHeaderId, db);
              mfPdu.save();
              break;
      
	
         // Acknowledge pdu
         case PDUTypeField.ACKNOWLEDGE:
              AcknowledgePduDML ackPdu = new AcknowledgePduDML((AcknowledgePdu)aPdu, disHeaderId, db);
              ackPdu.save();
              break;
	
         // StopFreeze pdu
         case PDUTypeField.STOPFREEZE:
              StopFreezePduDML sfPdu = new StopFreezePduDML((StopFreezePdu)aPdu, disHeaderId, db);
              sfPdu.save();
              break;

 	// StartResume pdu
         case PDUTypeField.STARTRESUME:
              StartResumePduDML srPdu = new StartResumePduDML((StartResumePdu)aPdu, disHeaderId, db);
              srPdu.save();
              break;
 	

         default:
            {
                // unrecognized PDU type.
              System.out.print("Unrecognized PDU type received -");
              System.out.println(aPdu.getPduType().shortValue());
            }


         }
         PDUTypeField aPduTypeField = new PDUTypeField();
//         msg("PDU Type = "+aPduTypeField.toString(aPdu.getPduType().shortValue()));
         disHeader = null;
//     }
      
 }


/***************************************************
 *  Gets all the parameters from the request
 **************************************************/   
 private void getParams(String str) {

       ht = HttpUtils.parseQueryString(str);
       action = ht.get("action").toString();
 }

/**
 * Handle the GET HTTP REQUEST Method 
 */

 public void doGet(HttpServletRequest req, HttpServletResponse res)
                    throws IOException {

     res.setContentType("text/html");
    // ServletOutputStream out = res.getOutputStream();
     PrintWriter out = new PrintWriter(res.getOutputStream());

     out.println("<html><title> PduEstateServlet Page </title>");
     out.println("<body bgcolor=white> Accepts PDUs by POST method in XML format...</body></html>");
     //insertTest();
     //getParams();
     /*     action = req.getParameter("action");
     if(action.equals("query")) {
         String t1 = req.getParameter("t1");
         String t2 = req.getParameter("t2");
         doQuery(t1,t2);
     }
     */
     out.flush();
     out.close();

     msg("Inside GET method");
 }

 private void doQuery(String t1, String t2) { 
 msg("Query for timestamps...");
 }

/*************************************
 * Test routine
 ************************************/ 
 private void insertTest() {
     Hashtable ht = new Hashtable();
     ht.put("id","1");
     ht.put("name","Tomy");
     ht.put("email","tapulika");
     ht.put("url","www.npac.syr.edu");
     ht.put("phone","315 443 4889");

     String sql = "insert into vrdata(id,name,email,url) values "+
				  "(?,?,?,?)";
     String tablename = "vrdata";
     msg("About to call insert ......");
     db.insert(ht,tablename,sql,5);
     msg("Test completed !");
     db.close();
 }

/**
 * Initialize the servlet, DB connection etc.
 */

 public void init(ServletConfig config) throws ServletException {
     super.init(config);
     db = new SQLUtil();
     db.open();
 }

/**
 * Clean up the servlet at the very end
 */
 
 public void destroy() {
     db.close();
 }

 private void msg(String str) {
     System.out.println(str);
 }

}

