/**
 * DetonationPdu database operations (DML's)
 */
package pduDB;

import java.sql.*;
import java.util.*;


import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

public class DetonationPduDML {

 private DetonationPdu dPdu;
 private SQLUtil db; //db connection
 private static String detonation_tablename = "dis_detonationpdu_table";
 private static String articulation_tablename = "dis_articulationparams_table";
 private static String[] detonation_cols = {
        "id", "disheader_id", "fireeID", "fireaID", "firesID", //1
        "targeteID","targetaID","targetsID","munitioneID", //2
        "munitionaID","munitionsID","eventID","eventaID","eventsID",//3
        "world_x","world_y","world_z",//4
        "burst_warhead","burst_fuse","burst_quantity","burst_rate",//5
        "etype_extra","etype_country","etype_kind","etype_specific","etype_category",//6
	"etype_subcategory","etype_domain", // 7
        "linearVelocity_x","linearVelocity_y","linearVelocity_z", //8
         "entityCoordinate_x","entityCoordinate_y","entityCoordinate_z",// 9
        "detonationResult","articulationParameterCount" //10
        }; //11
 private static String qMarks = "?,?,?,?,"+		//1				
                       		"?,?,?,?,"+//2
 				 "?,?,?,?,?,"+//3
				 "?,?,?,"+//4
				 "?,?,?,?,"+//5
				 "?,?,?,?,?,"+ //6
				 "?,?,"+ //7
				 "?,?,?,"+ //8
			         "?,?,?," +//9
 				"?,?"
				; //11
		
 private String sql = "insert into " + detonation_tablename;
 private Hashtable ht;
 private int disHeaderId;

 public DetonationPduDML() {
     System.out.println("DetonationPdu called");
 }
 public DetonationPduDML(DetonationPdu pdu, int disId, SQLUtil dbUtil) {
     dPdu = pdu;
     db = dbUtil;
     disHeaderId = disId;
    //dPdu.printValues(2,System.out);
     fillHash();
 }
 public DetonationPduDML(int disId, SQLUtil dbUtil) {
     dPdu = new DetonationPdu();
     db = dbUtil;
     ht = null;
     disHeaderId = disId;
     fillPdu();
 }

 private void fillHash(){
     ht = new Hashtable();
     ht.put("disheader_id",(new Integer(disHeaderId)).toString());
  
     EntityID eID = dPdu.getFiringEntityID();
     ht.put("fireeid",eID.getEntityID().toString());
     ht.put("fireaid",eID.getApplicationID().toString());
     ht.put("firesid",eID.getSiteID().toString());
  
     eID = dPdu.getTargetEntityID();
     ht.put("targeteid",eID.getEntityID().toString());
     ht.put("targetaid",eID.getApplicationID().toString());
     ht.put("targetsid",eID.getSiteID().toString());

     eID = dPdu.getMunitionID();
     ht.put("munitioneid",eID.getEntityID().toString());
     ht.put("munitionaid",eID.getApplicationID().toString());
     ht.put("munitionsid",eID.getSiteID().toString());
  
     EventID evID = dPdu.getEventID();
     ht.put("eventid",evID.getEventID().toString());
     ht.put("eventaid",evID.getApplicationID().toString());
     ht.put("eventsid",evID.getSiteID().toString());

     WorldCoordinate wcoord = dPdu.getLocationInWorldCoordinate();
     ht.put("world_x",(new Double(wcoord.getX())).toString());
     ht.put("world_y",(new Double(wcoord.getY())).toString());
     ht.put("world_z",(new Double(wcoord.getZ())).toString());

     BurstDescriptor burstd = dPdu.getBurstDescription();
     ht.put("burst_warhead",burstd.getWarhead().toString());
     ht.put("burst_fuse",burstd.getFuse().toString());
     ht.put("burst_quantity",burstd.getQuantity().toString());
     ht.put("burst_rate",burstd.getRate().toString());
     
     EntityType etype = burstd.getMunition();
     ht.put("etype_extra",etype.getExtra().toString());
     ht.put("etype_country",etype.getCountry().toString());
     ht.put("etype_kind",etype.getKind().toString());
     ht.put("etype_specific",etype.getSpecific().toString());
     ht.put("etype_category",etype.getCategory().toString());
     ht.put("etype_subcategory",etype.getSubCategory().toString());
     ht.put("etype_domain",etype.getDomain().toString());

     LinearVelocity lv = dPdu.getVelocity();
     ht.put("linearvelocity_z",
            (new Float(lv.getZ())).toString());
     ht.put("linearvelocity_y",
            (new Float(lv.getY())).toString());
     ht.put("linearvelocity_x",
            (new Float(lv.getX())).toString());

     EntityCoordinate ecoord = dPdu.getLocationInEntityCoordinates();
     ht.put("entitycoordinate_x",(new Float(ecoord.getX())).toString());
     ht.put("entitycoordinate_y",(new Float(ecoord.getY())).toString());
     ht.put("entitycoordinate_z",(new Float(ecoord.getZ())).toString());

     ht.put("detonationresult",dPdu.getDetonationResult().toString());
     ht.put("articulationparametercount",dPdu.articulationParameterCount().toString());
 }

 private void fillPdu(){
     String sql = "select * from "+detonation_tablename +
 	          " where disheader_id = " +disHeaderId;
     String articulationSql = "select * from "+ articulation_tablename +
                              " where pdu_id = ";
     int detonation_id = -1;
     try{
     ht = db.getRecord(sql);
     if(ht == null) {
         msg("Error !!! No entity state record found for this disheader ");
         return;
     }
     detonation_id = getInt("id");
     
     EntityID feID = new EntityID(getInt("firesid"),getInt("fireaid"),getInt("fireeid"));
     dPdu.setFiringEntityID(feID);
     
     EntityID teID = new EntityID(getInt("targetsid"),getInt("targetaid"),getInt("targeteid"));
     dPdu.setTargetEntityID(teID);
 
     EntityID meID = new EntityID(getInt("munitionsid"),getInt("munitionaid"),getInt("munitioneid"));
     dPdu.setMunitionID(meID);

     EventID ev = new EventID(getShort("eventid"),getShort("eventaid"),getShort("eventsid"));
     dPdu.setEventID(ev);


     WorldCoordinate wcoord = new WorldCoordinate(getDouble("world_x"),
                              getDouble("world_y"),getDouble("world_z"));
     dPdu.setLocationInWorldCoordinate(wcoord);
     
     BurstDescriptor burstd = new BurstDescriptor();
     EntityType mun = new EntityType();
     mun.setKind(getInt("etype_kind"));
     mun.setDomain(getInt("etype_domain"));
     mun.setCountry(getInt("etype_country"));
     mun.setCategory(getInt("etype_category"));
     mun.setSubCategory(getInt("etype_subcategory")); 
     mun.setSpecific(getInt("etype_specific"));
     mun.setExtra(getInt("etype_extra"));
     burstd.setMunition(mun);
     burstd.setWarhead(getInt("burst_warhead"));
     burstd.setFuse(getInt("burst_fuse")); 
     burstd.setQuantity(getInt("burst_quantity"));
     burstd.setRate(getInt("burst_rate"));
     dPdu.setBurstDescription(burstd);
     
     dPdu.setVelocity(getFloat("linearvelocity_x"),
       getFloat("linearvelocity_y"),getFloat("linearvelocity_z")); 
          
     dPdu.setLocationInEntityCoordinates(getFloat("entitycoordinate_x"),
          getFloat("entitycoordinate_y"),getFloat("entitycoordinate_z"));
     dPdu.setDetonationResult(getInt("detonationresult"));
     
        
     //now clear the hashtable ht for the new queries
     ht.clear();

     articulationSql = articulationSql + detonation_id;
     msg("articulation SQL = " + articulationSql);

     Vector v = db.getMultiRecords(articulationSql); 
            
     for (Enumeration e = v.elements() ; e.hasMoreElements() ;) {
         ht = (Hashtable)e.nextElement();
         if(ht == null) msg("Error !!! Articulation parameter is null !");
         dPdu.addArticulationParameter(getArticulationParam());         
     }
//     System.out.println("EPDU ************* ");
     //dPdu.printValues(2,System.out);
  //   System.out.println(" \n**********");

     }catch(Exception e) {msg(e.getMessage());}     
 }

 private String get(String name) {
     return ht.get(name).toString();
 }
 private int getInt(String name) {
     int i = 0;
     try {
         i = Integer.parseInt(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return i;
 }
 private float getFloat(String name) {
     float f = 0;
     try {
         f = (new Float(get(name))).floatValue();
     }catch(Exception e) {msg(e.getMessage());}
     return f;
 }
 
 private double getDouble(String name) {
     double d = 0.0;
     try {
         d = (new Double(get(name))).doubleValue();
     }catch(Exception e) { msg(e.getMessage()); }
     return d;
 }
 
 private short getShort(String name) {
     short s = 0;
     try {
         s = Short.parseShort(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return s;
 }

 private ArticulationParameter getArticulationParam(){
     
     ArticulationParameter artiparam = new ArticulationParameter();
     
     artiparam.setParameterTypeDesignator(getInt("ptdesignator"));
     artiparam.setChangeIndicator(getInt("changeindicator"));
     artiparam.setArticulationAttachmentID(getInt("articulationattachmentid"));
     artiparam.setParameterType(getInt("parametertype"));
     artiparam.setParameterValue(Long.parseLong(get("parametervalue")));
     return artiparam;
 }
 
 public void save(){
     int did = 0;
     sql = makeSQLStr(sql);
     //msg("opening database connection after making sql str");
     //db.open();
     //msg("sql string end product " +sql);
     db.insert(ht,detonation_tablename,sql,1); 
     //msg("Inserted on PDU...");
     try {
         did = Integer.parseInt(db.getMaxValue(detonation_tablename,"id"));
     }catch(Exception e) {
         msg("Error ! while getting Detonation Record Id ");
         msg(e.getMessage());
     }
//     msg("about to insert artiuculation params...");
     insertArticulationParameters(did);
     //db.close();

 }

 private String makeSQLStr(String s) {
     StringBuffer sb = new StringBuffer(s + "(");
     for(int i = 1; i < detonation_cols.length; i++) {
         if(i < detonation_cols.length-1)
	         sb.append(detonation_cols[i].toLowerCase() +",");
         else
			 sb.append(detonation_cols[i].toLowerCase() + ")");
     }
     sb.append(" values ( " + qMarks + " ) ");     
     return sb.toString();
 }

 private void  insertArticulationParameters(int detonationRecordId) {
     int articulationParamCount = 
             Integer.parseInt((String)ht.get("articulationparametercount"));
     for(int i =0; i< articulationParamCount; i++ ) {
         insertArticulationParameter(dPdu.getArticulationParameterAt(i),
                               detonationRecordId);
     }
 }

 private void insertArticulationParameter(ArticulationParameter aparam, 
		int detonationRecordId) {

     Hashtable ht = new Hashtable();
     String sql = "insert into "+articulation_tablename +
                  "( pdu_id, ptdesignator, changeindicator, "+
                  "parametertype, articulationattachmentid,parametervalue)"+
                  "values (?,?,?,?,?,?)";
     ht.put("pdu_id",(new Integer(detonationRecordId)).toString());
     ht.put("ptdesignator",aparam.getParameterTypeDesignator().toString());
     ht.put("changeindicator",aparam.getChangeIndicator().toString());
     ht.put("parametertype",aparam.getParameterType().toString());
     ht.put("articulationattachmentid",
                 aparam.getArticulationAttachmentID().toString());
     ht.put("parametervalue",(new Long(aparam.getParameterValue())).toString());
     db.insert(ht,articulation_tablename,sql,1); 
     ht = null;
 }


 private void msg(String str) {
     System.out.println(str);
 }
 
 public DetonationPdu getPDU() {
     return dPdu;
 }
}
