/**
 * EntityStatePdu database operations (DML's)
 */
package pduDB;

import java.sql.*;
import java.util.*;


import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

public class EntityStatePduDML {

 private EntityStatePdu ePdu;
 private SQLUtil db; //db connection
 private static String estate_tablename = "dis_entitystatepdu_table";
 private static String articulation_tablename = "dis_articulationparams_table";
 private static String[] estate_cols = { //make sure the order is the same as in table
        "id", "disheader_id", "eID", "aID", "sID",
    	"forceID", "articulationParameterCount", "etype_extra",
        "etype_country","etype_kind","etype_specific","etype_category",
		"etype_subcategory","etype_domain","aetype_extra","aetype_country",
		"aetype_kind","aetype_specific","aetype_category","aetype_subcategory",
		"aetype_domain","linearVelocity_x","linearVelocity_y",
		"linearVelocity_z","world_x","world_y","world_z","euler_theta",
		"euler_psi","euler_phi","entityAppearance","deadReckoningalgo",
		"linearAccel_x","linearAccel_y","linearAccel_z","angularVelo_x",
		"angularVelo_y","angularVelo_z","characterSet","eMarkings",
        "capabilities"};
 private static String qMarks = "?,?,?,?,"+						
                       				  "?,?,?,"+
 						 "?,?,?,?,"+
						 "?,?,?,?,"+
						 "?,?,?,?,"+
						 "?,?,?,"+
						 "?,?,?,?,?,"+
						 "?,?,?,?,"+
						 "?,?,?,?,"+
						 "?,?,?,?,"+
 						 "?";
		
 private String sql = "insert into " + estate_tablename;
 private Hashtable ht;
 private int disHeaderId;
 private int entityId = -1;

 public EntityStatePduDML() {
     System.out.println("EntitySatePdu called");
 }
 public EntityStatePduDML(EntityStatePdu pdu, int disId, SQLUtil dbUtil) {
     ePdu = pdu;
     db = dbUtil;
     disHeaderId = disId;
     fillHash();
 }
 public EntityStatePduDML(int disId, int eId, SQLUtil dbUtil) {
     ePdu = new EntityStatePdu();
     db = dbUtil;
     ht = null;
     disHeaderId = disId;
     entityId = eId;
     fillPdu();
 }
 private void fillHash(){
     ht = new Hashtable();
     ht.put("disheader_id",(new Integer(disHeaderId)).toString());
     EntityID eID = ePdu.getEntityID();
     ht.put("eid",eID.getEntityID().toString());
     ht.put("aid",eID.getApplicationID().toString());
     ht.put("sid",eID.getSiteID().toString());
     ht.put("forceid",ePdu.getForceID().toString());
     ht.put("articulationparametercount",
                      ePdu.articulationParameterCount().toString());
     ht.put("etype_extra",ePdu.getEntityTypeExtra().toString());
     ht.put("etype_country",ePdu.getEntityTypeCountry().toString());
     ht.put("etype_kind",ePdu.getEntityTypeKind().toString());
     ht.put("etype_specific",ePdu.getEntityTypeSpecific().toString());
     ht.put("etype_category",ePdu.getEntityTypeCategory().toString());
     ht.put("etype_subcategory",ePdu.getEntityTypeSubcategory().toString());
     ht.put("etype_domain",ePdu.getEntityTypeDomain().toString());
     ht.put("aetype_extra",ePdu.getAlternativeEntityTypeExtra().toString());
     ht.put("aetype_country",ePdu.getAlternativeEntityTypeCountry().toString());
     ht.put("aetype_kind",ePdu.getAlternativeEntityTypeKind().toString());
     ht.put("aetype_specific",
                     ePdu.getAlternativeEntityTypeSpecific().toString());
     ht.put("aetype_category",
                     ePdu.getAlternativeEntityTypeCategory().toString());
     ht.put("aetype_subcategory",
                     ePdu.getAlternativeEntityTypeSubcategory().toString());
     ht.put("aetype_domain",ePdu.getEntityTypeDomain().toString());

     ht.put("linearvelocity_z",
            (new Float(ePdu.getEntityLinearVelocityZ())).toString());
     //System.out.println("Velocity Z = "+ePdu.getEntityLinearVelocityZ());
     ht.put("linearvelocity_y",
            (new Float(ePdu.getEntityLinearVelocityY())).toString());
     ht.put("linearvelocity_x",
            (new Float(ePdu.getEntityLinearVelocityX())).toString());

     ht.put("world_x",(new Double(ePdu.getEntityLocationX())).toString());
     ht.put("world_y",(new Double(ePdu.getEntityLocationY())).toString());
     ht.put("world_z",(new Double(ePdu.getEntityLocationZ())).toString());

     ht.put("euler_theta",
            (new Float(ePdu.getEntityOrientationTheta())).toString());
     ht.put("euler_psi", 
            (new Float(ePdu.getEntityOrientationPsi())).toString());
     ht.put("euler_phi",
             (new Float(ePdu.getEntityOrientationPhi())).toString());

     ht.put("entityappearance",ePdu.getEntityAppearance().toString());

     ht.put("deadreckoningalgo",ePdu.getDeadReckoningAlgorithm().toString());

     ht.put("linearaccel_z",
            (new Float(ePdu.getEntityLinearAccelerationZ())).toString());
     ht.put("linearaccel_y",
            (new Float(ePdu.getEntityLinearAccelerationY())).toString());
     ht.put("linearaccel_x",
            (new Float(ePdu.getEntityLinearAccelerationX())).toString());

     ht.put("angularvelo_z",
            (new Float(ePdu.getEntityAngularVelocityZ())).toString());
     ht.put("angularvelo_y",
            (new Float(ePdu.getEntityAngularVelocityY())).toString());
     ht.put("angularvelo_x",
            (new Float(ePdu.getEntityAngularVelocityX())).toString());

     ht.put("characterset",ePdu.getCharacterSet().toString());
     ht.put("emarkings",ePdu.getMarking());
     ht.put("capabilities",ePdu.getCapabilities().toString());

  //   System.out.println("HT in fill hash = "+ ht.toString());
 
 }

 private void fillPdu(){
     String sql = "select * from "+estate_tablename +
 	          " where disheader_id = " +disHeaderId;
    // if(entityId > 0) {
      //   sql = sql + " and eid = "+entityId;
     //}
     sql = sql + " order by id";
 
     String articulationSql = "select * from "+ articulation_tablename +
                              " where pdu_id = ";
     int estate_id = -1;
     try{
     ht = db.getRecord(sql);
     if(ht == null) {
         msg("Error !!! No entity state record found for this disheader ");
         return;
     }
     estate_id = getInt("id");
   
     EntityID eID = new EntityID(getInt("sid"),getInt("aid"),getInt("eid"));
     ePdu.setEntityID(eID);
     ePdu.setForceID(getInt("forceid"));
 
     ePdu.setEntityTypeExtra(getInt("etype_extra"));
     ePdu.setEntityTypeCountry(getInt("etype_country"));
     ePdu.setEntityTypeKind(getInt("etype_kind"));
     ePdu.setEntityTypeCategory(getInt("etype_category"));
     ePdu.setEntityTypeSubcategory(getInt("etype_subcategory")); 
     ePdu.setEntityDomain(getInt("etype_domain"));
     ePdu.setEntityTypeSpecific(getInt("etype_specific"));
 
     ePdu.setAlternativeEntityTypeExtra(getInt("aetype_extra"));
     ePdu.setAlternativeEntityCountry(getInt("aetype_country"));
     ePdu.setAlternativeEntityTypeKind(getInt("aetype_kind"));
     ePdu.setAlternativeEntityTypeCategory(getInt("aetype_category"));
     ePdu.setAlternativeEntityTypeSubcategory(getInt("aetype_subcategory")); 
     ePdu.setAlternativeEntityDomain(getInt("aetype_domain"));
     ePdu.setAlternativeEntityTypeSpecific(getInt("aetype_specific"));
 
     ePdu.setEntityLinearVelocityX(getFloat("linearvelocity_x"));
     ePdu.setEntityLinearVelocityY(getFloat("linearvelocity_y"));
     ePdu.setEntityLinearVelocityZ(getFloat("linearvelocity_z"));  

          
     ePdu.setEntityLocationX(getDouble("world_x"));
     ePdu.setEntityLocationY(getDouble("world_y"));
     ePdu.setEntityLocationZ(getDouble("world_z"));

     ePdu.setEntityOrientationPsi(getFloat("euler_psi"));
     ePdu.setEntityOrientationPhi(getFloat("euler_phi"));
     ePdu.setEntityOrientationTheta(getFloat("euler_theta"));

     ePdu.setEntityAppearance(getInt("entityappearance"));

     ePdu.setDeadReckoningAlgorithm(getInt("deadreckoningalgo"));
     // have to get this from db actually...but for time being 
     // this is okay..
     byte[] deadReckoningParams = new byte[11];
     for(int idx = 0; idx < 11; idx++)
         deadReckoningParams[idx] = 0;

     ePdu.setDeadReckoningParameters(deadReckoningParams);

     ePdu.setEntityLinearAcceleration(getFloat("linearaccel_x"), 
              getFloat("linearaccel_y"), getFloat("linearaccel_z"));
      
     ePdu.setEntityAngularVelocity(getFloat("angularvelo_x"),
              getFloat("angularvelo_y"), getFloat("angularvelo_z"));
 
     ePdu.setCharacterSet(getInt("characterset"));
     ePdu.setMarking(get("emarkings"));
     ePdu.setCapabilities(getInt("capabilities"));     

     //now clear the hashtable ht for the new queries
     ht.clear();

     articulationSql = articulationSql + estate_id;
     //msg("articulation SQL = " + articulationSql);

     Vector v = db.getMultiRecords(articulationSql); 
            
     for (Enumeration e = v.elements() ; e.hasMoreElements() ;) {
         ht = (Hashtable)e.nextElement();
         if(ht == null) msg("Error !!! Articulation parameter is null !");
         ePdu.addArticulationParameter(getArticulationParam());         
     }
     //System.out.println("EPDU ************* ");
     //ePdu.printValues(2,System.out);
     //System.out.println(" \n**********");

     }catch(Exception e) {msg(e.getMessage());}     
 }

 private String get(String name) {
     return ht.get(name).toString();
 }
 private int getInt(String name) {
     int i = 0;
     try {
         i = Integer.parseInt(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return i;
 }
 private float getFloat(String name) {
     float f = 0;
     try {
         f = (new Float(get(name))).floatValue();
     }catch(Exception e) {msg(e.getMessage());}
     return f;
 }
 
 private double getDouble(String name) {
     double d = 0.0;
     try {
         d = (new Double(get(name))).doubleValue();
     }catch(Exception e) { msg(e.getMessage()); }
     return d;
 }

 private ArticulationParameter getArticulationParam(){
     
     ArticulationParameter artiparam = new ArticulationParameter();
     
     artiparam.setParameterTypeDesignator(getInt("ptdesignator"));
     artiparam.setChangeIndicator(getInt("changeindicator"));
     artiparam.setArticulationAttachmentID(getInt("articulationattachmentid"));
     artiparam.setParameterType(getInt("parametertype"));
     artiparam.setParameterValue(Long.parseLong(get("parametervalue")));
     return artiparam;
 }
 
 public void save(){
     int esid = 0;
     sql = makeSQLStr(sql);
     //msg("opening database connection after making sql str");
     //db.open();
     //msg("sql string end product " +sql);
     db.insert(ht,estate_tablename,sql,1); 
     //msg("Inserted on PDU...");
     try {
         esid = Integer.parseInt(db.getMaxValue(estate_tablename,"id"));
     }catch(Exception e) {
         msg("Error ! while getting Entity State Record Id ");
         msg(e.getMessage());
     }
     //msg("about to insert artiuculation params...");
     insertArticulationParameters(esid);
     //db.close();

 }

 private String makeSQLStr(String s) {
     StringBuffer sb = new StringBuffer(s + "(");
     for(int i = 1; i < estate_cols.length; i++) {
         if(i < estate_cols.length-1)
	         sb.append(estate_cols[i].toLowerCase() +",");
         else
			 sb.append(estate_cols[i].toLowerCase() + ")");
     }
     sb.append(" values ( " + qMarks + " ) ");     
     return sb.toString();
 }

 private void  insertArticulationParameters(int eStateRecordId) {
     int articulationParamCount = 
             Integer.parseInt((String)ht.get("articulationparametercount"));
     for(int i =0; i< articulationParamCount; i++ ) {
         insertArticulationParameter(ePdu.getArticulationParameterAt(i),
                               eStateRecordId);
     }
     ht = null;
 }

 private void insertArticulationParameter(ArticulationParameter aparam, 
		int eStateRecordId) {
     
     Hashtable aht = new Hashtable();
     String sql = "insert into "+articulation_tablename +
                  "( pdu_id, ptdesignator, changeindicator, "+
                  "parametertype, articulationattachmentid,parametervalue)"+
                  "values (?,?,?,?,?,?)";
     aht.put("pdu_id",(new Integer(eStateRecordId)).toString());
     aht.put("ptdesignator",aparam.getParameterTypeDesignator().toString());
     aht.put("changeindicator",aparam.getChangeIndicator().toString());
     aht.put("parametertype",aparam.getParameterType().toString());
     aht.put("articulationattachmentid",
                 aparam.getArticulationAttachmentID().toString());
     aht.put("parametervalue",(new Long(aparam.getParameterValue())).toString());
     db.insert(aht,articulation_tablename,sql,1); 
     aht = null;
 }


 private void msg(String str) {
     System.out.println(str);
 }
 
 public EntityStatePdu getPDU() {
     return ePdu;
 }
}
