/**
 * FirePdu database operations (DML's)
 */
package pduDB;

import java.sql.*;
import java.util.*;


import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

public class FirePduDML {

 private FirePdu fPdu;
 private SQLUtil db; //db connection
 private static String fire_tablename = "dis_firepdu_table";
 
 private static String[] fire_cols = {
        "id", "disheader_id", "fireeID", "fireaID", "firesID", //1
        "targeteID","targetaID","targetsID","munitioneID", //2
        "munitionaID","munitionsID","eventID","eventaID","eventsID",//3
        "fireMissionIndex","world_x","world_y","world_z",//4
        "burst_warhead","burst_fuse","burst_quantity","burst_rate",//5
       "etype_extra","etype_country","etype_kind","etype_specific","etype_category",//6
	"etype_subcategory","etype_domain", // 7
        "linearVelocity_x","linearVelocity_y","linearVelocity_z", //8
        "fireRange" //9
        }; //10
 private static String qMarks = "?,?,?,?,"+		//1				
                       				 "?,?,?,?,"+//2
 						 "?,?,?,?,?,"+//3
						 "?,?,?,?,"+//4
						 "?,?,?,?,"+//5
						 "?,?,?,?,?,"+ //6
						 "?,?,"+ //7
						 "?,?,?," +//8
                                                 "?" //9
 						 ; //11
		
 private String sql = "insert into " + fire_tablename;
 private Hashtable ht;
 private int disHeaderId;

 public FirePduDML() {
     System.out.println("FirePdu called");
 }
 public FirePduDML(FirePdu pdu, int disId, SQLUtil dbUtil) {
     fPdu = pdu;
     db = dbUtil;
     disHeaderId = disId;
     fillHash();
     //fPdu.printValues(2,System.out);
 }
 public FirePduDML(int disId, SQLUtil dbUtil) {
     fPdu = new FirePdu();
     db = dbUtil;
     ht = null;
     disHeaderId = disId;
     fillPdu();
 }

 private void fillHash(){
     ht = new Hashtable();
     ht.put("disheader_id",(new Integer(disHeaderId)).toString());
  
     EntityID eID = fPdu.getFiringEntityID();
     ht.put("fireeid",eID.getEntityID().toString());
     ht.put("fireaid",eID.getApplicationID().toString());
     ht.put("firesid",eID.getSiteID().toString());
  
     eID = fPdu.getTargetEntityID();
     ht.put("targeteid",eID.getEntityID().toString());
     ht.put("targetaid",eID.getApplicationID().toString());
     ht.put("targetsid",eID.getSiteID().toString());

     eID = fPdu.getMunitionID();
     ht.put("munitioneid",eID.getEntityID().toString());
     ht.put("munitionaid",eID.getApplicationID().toString());
     ht.put("munitionsid",eID.getSiteID().toString());
  
     EventID evID = fPdu.getEventID();
     ht.put("eventid",evID.getEventID().toString());
     ht.put("eventaid",evID.getApplicationID().toString());
     ht.put("eventsid",evID.getSiteID().toString());

     ht.put("firemissionindex",fPdu.getFireMissionIndex().toString());

     WorldCoordinate wcoord = fPdu.getLocationInWorldCoordinate();
     ht.put("world_x",(new Double(wcoord.getX())).toString());
     ht.put("world_y",(new Double(wcoord.getY())).toString());
     ht.put("world_z",(new Double(wcoord.getZ())).toString());

     BurstDescriptor burstd = fPdu.getBurstDescriptor();
     ht.put("burst_warhead",burstd.getWarhead().toString());
     ht.put("burst_fuse",burstd.getFuse().toString());
     ht.put("burst_quantity",burstd.getQuantity().toString());
     ht.put("burst_rate",burstd.getRate().toString());
     
     EntityType etype = burstd.getMunition();
     ht.put("etype_extra",etype.getExtra().toString());
     ht.put("etype_country",etype.getCountry().toString());
     ht.put("etype_kind",etype.getKind().toString());
     ht.put("etype_specific",etype.getSpecific().toString());
     ht.put("etype_category",etype.getCategory().toString());
     ht.put("etype_subcategory",etype.getSubCategory().toString());
     ht.put("etype_domain",etype.getDomain().toString());

     LinearVelocity lv = fPdu.getVelocity();
     ht.put("linearvelocity_z",
            (new Float(lv.getZ())).toString());
     ht.put("linearvelocity_y",
            (new Float(lv.getY())).toString());
     ht.put("linearvelocity_x",
            (new Float(lv.getX())).toString());

     ht.put("firerange",(new Float(fPdu.getRange())).toString());
     

  //   System.out.println("HT in fill hash = "+ ht.toString());
 
 }

 private void fillPdu(){
     String sql = "select * from "+fire_tablename +
 	          " where disheader_id = " +disHeaderId;
   
     int pdu_id = -1;
     try{
     ht = db.getRecord(sql);
     if(ht == null) {
         msg("Error !!! No entity state record found for this disheader ");
         return;
     }
     pdu_id = getInt("id");
     System.out.println("pdu fire id = "+ pdu_id);
   
     EntityID feID = new EntityID(getInt("firesid"),getInt("fireaid"),getInt("fireeid"));
     fPdu.setFiringEntityID(feID);
     
     EntityID teID = new EntityID(getInt("targetsid"),getInt("targetaid"),getInt("targeteid"));
     fPdu.setTargetEntityID(teID);
 
     EntityID meID = new EntityID(getInt("munitionsid"),getInt("munitionaid"),getInt("munitioneid"));
     fPdu.setMunitionID(meID);

     EventID ev = new EventID(getShort("eventid"),getShort("eventaid"),getShort("eventsid"));
     fPdu.setEventID(ev);

     fPdu.setFireMissionIndex(getInt("firemissionindex"));
     msg("set tille fire mission idex");

     WorldCoordinate wcoord = new WorldCoordinate(getDouble("world_x"),
                              getDouble("world_y"),getDouble("world_z"));
     fPdu.setLocationInWorldCoordinate(wcoord);
     
     msg("location is set");
     BurstDescriptor burstd = new BurstDescriptor();
     EntityType mun = new EntityType();
     mun.setKind(getInt("etype_kind"));
     mun.setDomain(getInt("etype_domain"));
     mun.setCountry(getInt("etype_country"));
     mun.setCategory(getInt("etype_category"));
     mun.setSubCategory(getInt("etype_subcategory")); 
     mun.setSpecific(getInt("etype_specific"));
     mun.setExtra(getInt("etype_extra"));
     burstd.setMunition(mun);
     burstd.setWarhead(getInt("burst_warhead"));
     burstd.setFuse(getInt("burst_fuse")); 
     burstd.setQuantity(getInt("burst_quantity"));
     burstd.setRate(getInt("burst_rate"));
     fPdu.setBurstDescriptor(burstd);
 
     msg("burst descriptor is set");
     
     fPdu.setVelocity(getFloat("linearvelocity_x"),
       getFloat("linearvelocity_y"),getFloat("linearvelocity_z")); 
        
     msg("velocity is set");
   
     fPdu.setRange(getFloat("firerange"));

     }catch(Exception e) {msg(e.getMessage());}     
 }

 private String get(String name) {
     return ht.get(name).toString();
 }
 private int getInt(String name) {
     int i = 0;
     try {
         i = Integer.parseInt(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return i;
 }
 private float getFloat(String name) {
     float f = 0;
     try {
         f = (new Float(get(name))).floatValue();
     }catch(Exception e) {msg(e.getMessage());}
     return f;
 }
 
 private double getDouble(String name) {
     double d = 0.0;
     try {
         d = (new Double(get(name))).doubleValue();
     }catch(Exception e) { msg(e.getMessage()); }
     return d;
 }
 
 private short getShort(String name) {
     short s = 0;
     try {
         s = Short.parseShort(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return s;
 }
 
 
 public void save(){
     sql = makeSQLStr(sql);
     //msg("opening database connection after making sql str");
     //db.open();
     //msg("sql string end product " +sql);
     db.insert(ht,fire_tablename,sql,1); 
     //msg("Inserted on PDU...");
    
     //db.close();

 }

 private String makeSQLStr(String s) {
     StringBuffer sb = new StringBuffer(s + "(");
     for(int i = 1; i < fire_cols.length; i++) {
         if(i < fire_cols.length-1)
	         sb.append(fire_cols[i].toLowerCase() +",");
         else
			 sb.append(fire_cols[i].toLowerCase() + ")");
     }
     sb.append(" values ( " + qMarks + " ) ");     
     return sb.toString();
 }

 private void msg(String str) {
     System.out.println(str);
 }
 
 public FirePdu getPDU() {
     return fPdu;
 }
}
