/**
 * MinePdu database operations (DML's)
 */
package pduDB;

import java.sql.*;
import java.util.*;


import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

import iwt.mine.*;

public class MinePduDML {

 private MinePdu mPdu;
 private SQLUtil db; //db connection
 private static String mine_tablename = "dis_minepdu_table";
 private static String location_tablename = "dis_location_table";
 private static String orientation_tablename = "dis_orientation_table";

 private static String[] mine_cols = {
        "id", "disheader_id", "eID", "aID", "sID","sequence","quiescent",
    	"etype_extra","etype_country","etype_kind","etype_specific","etype_category",
	"etype_subcategory","etype_domain","fuzetype","numberofmine","world_x","world_y","world_z",
        "burialdepthflag","temperatureflag","orientationflag"
        };
 private static String qMarks = "?,?,?,?,?,?,"+						
                       		"?,?,?,?,?,"+
				"?,?,?,?,?,?,?,"+
 				"?,?,?";
		
 private String sql = "insert into " + mine_tablename;
 private Hashtable ht;
 private int disHeaderId;

//float arrays and flags
 private boolean bDepthFlag ;
 private float[] bDepth = null;
 private boolean temperatureFlag ;
 private float[] temperatures = null;
 private boolean orientationFlag ;
 private EulerAngle[] orientations = null;
 private VectorRecord[] positionVectors = null;

 public MinePduDML() {
     System.out.println("MinePdu called");
 }
 public MinePduDML(MinePdu pdu, int disId, SQLUtil dbUtil) {
     mPdu = pdu;
     db = dbUtil;
     disHeaderId = disId;
     fillHash();
 }
 public MinePduDML(int disId, SQLUtil dbUtil) {
     mPdu = new MinePdu();
     db = dbUtil;
     ht = null;
     disHeaderId = disId;
     fillPdu();
 }

 private void fillHash(){
     int i = 0;
     ht = new Hashtable();
     ht.put("disheader_id",(new Integer(disHeaderId)).toString());
     EntityID eID = mPdu.getMineEntityID();
     ht.put("eid",eID.getEntityID().toString());
     ht.put("aid",eID.getApplicationID().toString());
     ht.put("sid",eID.getSiteID().toString());
 
     TSequence tseq = mPdu.getQSequence();
     ht.put("sequence",(new Short(tseq.getSequence())).toString());
     ht.put("quiescent",(new Boolean(tseq.getQuiescent())).toString());

     EntityType etype = mPdu.getMunition();
     ht.put("etype_extra",etype.getExtra().toString());
     ht.put("etype_country",etype.getCountry().toString());
     ht.put("etype_kind",etype.getKind().toString());
     ht.put("etype_specific",etype.getSpecific().toString());
     ht.put("etype_category",etype.getCategory().toString());
     ht.put("etype_subcategory",etype.getSubCategory().toString());
     ht.put("etype_domain",etype.getDomain().toString());
      
     ht.put("fuzetype",mPdu.getFuzeType().toString());
     ht.put("numberofmine",mPdu.getNumOfMines().toString());

     WorldCoordinate wcoord = mPdu.getRefLocation();
     ht.put("world_x",(new Double(wcoord.getX())).toString());
     ht.put("world_y",(new Double(wcoord.getY())).toString());
     ht.put("world_z",(new Double(wcoord.getZ())).toString());
 
     MineDataFlag mdFlag = mPdu.getMineDataFlag();
     int mineNum = mPdu.getNumOfMines().intValue();
     System.out.println("Inside MinePduDML:Total # of Mines = "+ mineNum);

     positionVectors = new VectorRecord[mineNum];
     for(i =0; i< mineNum; i++)
         positionVectors[i] = mPdu.getLocationAt(i);
    

     bDepthFlag = mdFlag.getBurialDepthFlag();
     ht.put("burialdepthflag",(new Boolean(bDepthFlag)).toString());
     if(bDepthFlag) {
         bDepth = new float[mineNum];
         for(i =0; i<mineNum; i++) 
	     bDepth[i] = mPdu.getBurialDepthAt(i);
     }
     temperatureFlag = mdFlag.getTemperature();
     ht.put("temperatureflag",(new Boolean(temperatureFlag)).toString());
     if(temperatureFlag) {
         temperatures = new float[mineNum];
         for(i =0; i<mineNum; i++) 
	     temperatures[i] = mPdu.getTemperatureAt(i);
     }
     orientationFlag = mdFlag.getOrientationFlag();
     ht.put("orientationflag",(new Boolean(orientationFlag)).toString());
     if(orientationFlag) {
         orientations = new EulerAngle[mineNum];
         for(i =0; i<mineNum; i++) 
	     orientations[i] = mPdu.getOrientationAt(i);
     }
  //   System.out.println("HT in fill hash = "+ ht.toString());
 
 }

 private void fillPdu(){
     String sql = "select * from "+mine_tablename +
 	          " where disheader_id = " +disHeaderId;
     String sqlPos = "select * from "+location_tablename +
		     " where pdu_id = ";
     String sqlOrient = "select * from "+orientation_tablename +
           		" where pdu_id = ";
    
     int mine_id = -1;
     try{
     ht = db.getRecord(sql);
     if(ht == null) {
         msg("Error !!! No mine pdu record found for this disheader ");
         return;
     }
     mine_id = getInt("id");
   
     EntityID eID = new EntityID(getInt("sid"),getInt("aid"),getInt("eid"));
     mPdu.setMineEntityID(eID);
   
     TSequence tseq = new TSequence((new Boolean(get("quiescent"))).booleanValue(),
                                 getShort("sequence"));  
     mPdu.setQSequence(tseq);
  
     EntityType etype = new EntityType();
     etype.setExtra(getInt("etype_extra"));
     etype.setCountry(getInt("etype_country"));
     etype.setKind(getInt("etype_kind"));
     etype.setCategory(getInt("etype_category"));
     etype.setSubCategory(getInt("etype_subcategory")); 
     etype.setDomain(getInt("etype_domain"));
     etype.setSpecific(getInt("etype_specific"));
     mPdu.setMunition(etype);
 
     mPdu.setFuzeType(new UnsignedShort(getShort("fuzetype")));
     mPdu.setNumOfMines(new UnsignedShort(getShort("numberofmine")));     

     WorldCoordinate wcoord = new WorldCoordinate();
     wcoord.setX(getDouble("world_x"));
     wcoord.setY(getDouble("world_y"));
     wcoord.setZ(getDouble("world_z"));
     mPdu.setRefLocation(wcoord);

     MineDataFlag mdFlag = new  MineDataFlag(getBoolean("orientationflag"),
 		getBoolean("burialdepthflag"), getBoolean("temperatureflag"),0);
     mPdu.setMineDataFlag(mdFlag);

     //now clear the hashtable ht for the new queries
     ht.clear();

     //set positions, orientation, temperature and burialdepth
     sqlPos = sqlPos + mine_id + " order by id";
     //msg("SQL inside minepdudml - fillpdu = "+sqlPos);
     Vector v = db.getMultiRecords(sqlPos);
     int i =0;
     int cnt = v.size();
     for (Enumeration e = v.elements() ; e.hasMoreElements() ;) {
         ht = (Hashtable)e.nextElement();
         if(ht == null) msg("Error !!! Location vector is null !");
         mPdu.setLocationAt(i,getLocationVector());
         i++;         
     }
  //   System.out.println("Vector's have been set - "+cnt);
     if(orientationFlag) {
         sqlOrient = sqlOrient + mine_id + " order by id";
         v = db.getMultiRecords(sqlOrient);
         i =0;
         for (Enumeration e = v.elements() ; e.hasMoreElements() ;) {
             ht = (Hashtable)e.nextElement();
             if(ht == null) msg("Error !!! Orientation vector is null !");
             mPdu.setOrientationAt(i,getOrientation());
             i++;         
         }
      } //end of if

      if(bDepthFlag) {
      }//do this
      if(temperatureFlag) {
      }//do this

     }catch(Exception e) {msg(e.getMessage());}     
 }

 private String get(String name) {
     return ht.get(name).toString();
 }
 private int getInt(String name) {
     int i = 0;
     try {
         i = Integer.parseInt(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return i;
 }
 
 private short getShort(String name) {
     short s = 0;
     try {
         s = Short.parseShort(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return s;
 }
 
 private double getDouble(String name) {
     double d = 0.0;
     try {
         d = (new Double(get(name))).doubleValue();
     }catch(Exception e) { msg(e.getMessage()); }
     return d;
 }
 
 private boolean getBoolean(String name) {
     boolean b = false; 
     try {
         b = (new Boolean(get(name))).booleanValue();
     }catch(Exception e) { msg(e.getMessage()); }
     return b;
 }

 private float getFloat(String name) {
     float f = 0; 
     try {
         f = (new Float(get(name))).floatValue();
     }catch(Exception e) { msg(e.getMessage()); }
     return f;
 }
 public void save(){
     String mid = "";
    
     sql = makeSQLStr(sql);
     //msg("opening database connection after making sql str");
     //db.open();
     //msg("sql string end product " +sql);
     db.insert(ht,mine_tablename,sql,1); 
     //msg("Inserted on PDU...");

     try {
         mid = db.getMaxValue(mine_tablename,"id");
     }catch(Exception e) {
         msg("Error ! while getting Mine Pdu Record Id ");
         msg(e.getMessage());
     }

     ht.clear();   
     insertPositionVectors(mid);
     if(orientationFlag) insertOrientationValues(mid);
     if(bDepthFlag) insertBurialDepthValues(mid);
     if(temperatureFlag) insertTemperatureValues(mid);
 }

 private String makeSQLStr(String s) {
     StringBuffer sb = new StringBuffer(s + "(");
     for(int i = 1; i < mine_cols.length; i++) {
         if(i < mine_cols.length-1)
	         sb.append(mine_cols[i].toLowerCase() +",");
         else
			 sb.append(mine_cols[i].toLowerCase() + ")");
     }
     sb.append(" values ( " + qMarks + " ) ");     
     return sb.toString();
 }

 private void insertPositionVectors(String minePduId){
     String sqlPos = "insert into "+location_tablename +
	 " (pdu_id, xcoord, ycoord, zcoord) values (?,?,?,?)";   
     //msg("came inside insert Position vectors"); 
     VectorRecord vRec = new VectorRecord();
     msg("pos vec length=");
     msg(positionVectors.length);

     for(int i=0; i<positionVectors.length; i++) {
         vRec = positionVectors[i];
         //System.out.println("x="+vRec.getX());
         ht.put("pdu_id",minePduId);
         ht.put("xcoord",Float.toString(vRec.getX()));
         ht.put("ycoord",Float.toString(vRec.getY()));
         ht.put("zcoord",Float.toString(vRec.getZ()));
         try{ 
             db.insert(ht,location_tablename,sqlPos,1);       
         }catch(Exception e){e.printStackTrace(System.out);}
         ht.clear();
     }
 }

 private void insertOrientationValues(String minePduId){

     String sqlOrient = "insert into "+orientation_tablename +
	 " (pdu_id, psi, phi, theta) values (?,?,?,?)";   
      
     EulerAngle eAng;
     for(int i=0; i<orientations.length; i++) {
         eAng = orientations[i];
         ht.put("pdu_id",minePduId);
         ht.put("psi",Float.toString(eAng.getPsi()));
         ht.put("phi",Float.toString(eAng.getPhi()));
         ht.put("theta",Float.toString(eAng.getTheta())); 
         db.insert(ht,orientation_tablename,sqlOrient,1); 
         ht.clear();      
     }
    // eAng = null;
 }

 private void insertBurialDepthValues(String minePduId){
 }
 private void insertTemperatureValues(String minePduId){
 }

 private VectorRecord getLocationVector() {
     VectorRecord locRec = new VectorRecord(getFloat("xcoord"),
	getFloat("ycoord"), getFloat("zcoord"));
     return locRec;
 }

 private EulerAngle getOrientation() {
     EulerAngle eAng = new EulerAngle(getFloat("psi"),
	getFloat("phi"), getFloat("theta"));
     return eAng;
 }
 private void msg(String str) {
     System.out.println(str);
 }
 private void msg(int i) {
     System.out.println(i);
 }
 public MinePdu getPDU() {
     return mPdu;
 }
}
