/**
 * MinefieldPdu database operations (DML's)
 */
package pduDB;

import java.sql.*;
import java.util.*;


import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

import iwt.mine.*;

public class MinefieldPduDML {

 private MinefieldPdu mfPdu;
 private SQLUtil db; //db connection
 private static String minefield_tablename = "dis_minefieldpdu_table";
 private static String location_tablename = "dis_location_table";
 private static String[] minefield_cols = { //make sure this is in the order
  					    // of the columns in the table
        "id", "disheader_id", "eid", "aid",
    	"sid", "quiescent","sequence",
        "appearance","numberofvertices","forceid"
        };
 private static String qMarks = "?,?,?,"+						
          			"?,?,?,"+
 			  	"?,?,?";
		
 private String sql = "insert into " + minefield_tablename;
 private Hashtable ht;
 private int disHeaderId;

 private WorldCoordinate[] wcoords = null;

 public MinefieldPduDML() {
     System.out.println("MinefieldPdu called");
 }
 public MinefieldPduDML(MinefieldPdu pdu, int disId, SQLUtil dbUtil) {
     mfPdu = pdu;
     db = dbUtil;
     disHeaderId = disId;
     fillHash();
 }
 public MinefieldPduDML(int disId, SQLUtil dbUtil) {
     mfPdu = new MinefieldPdu();
     db = dbUtil;
     ht = null;
     disHeaderId = disId;
     fillPdu();
 }

 private void fillHash(){
     ht = new Hashtable();
     ht.put("disheader_id",(new Integer(disHeaderId)).toString());
     EntityID eID = mfPdu.getMinefieldEntityID();
     ht.put("eid",eID.getEntityID().toString());
     ht.put("aid",eID.getApplicationID().toString());
     ht.put("sid",eID.getSiteID().toString());
     ht.put("forceid",mfPdu.getForceID().toString());
    
     TSequence tseq = mfPdu.getQSequence();
     ht.put("sequence",(new Short(tseq.getSequence())).toString());
     ht.put("quiescent",(new Boolean(tseq.getQuiescent())).toString());
     ht.put("appearance",mfPdu.getAppearance().toString());
     ht.put("numberofvertices",mfPdu.getNumOfVertices().toString());
     int verticeNum =(int) mfPdu.getNumOfVertices().longValue();
 
     if(verticeNum > 0) {
         msg("Vertice number is ");msg(verticeNum);
         wcoords = new WorldCoordinate[verticeNum];
         for(int i=0; i<verticeNum; i++){ 
	     wcoords[i] = mfPdu.getRefLocationVerticeAt(i);
             //if(wcoords[i] !=null)msg("Wcoord is not null");
            // wcoords[i].printValues(2,System.out);
         }
     }
 
  //   System.out.println("HT in fill hash = "+ ht.toString());
 
 }

 private void fillPdu(){
     String sql = "select * from "+minefield_tablename +
 	          " where disheader_id = " +disHeaderId;
     String sqlCoord = "select * from "+location_tablename +
                       " where pdu_id = ";
     int minefield_id = -1;
     try{
     ht = db.getRecord(sql);
     if(ht == null) {
         msg("Error !!! No entity state record found for this disheader ");
         return;
     }
     minefield_id = getInt("id");
   
     EntityID eID = new EntityID(getInt("sid"),getInt("aid"),getInt("eid"));
     mfPdu.setMinefieldEntityID(eID);
     mfPdu.setForceID(new UnsignedByte(getInt("forceid")));
   
     TSequence tseq = new TSequence((new Boolean(get("quiescent"))).booleanValue(),
                                 getShort("sequence"));  
     mfPdu.setQSequence(tseq);
     mfPdu.setAppearance(new UnsignedShort(getShort("appearance")));
     mfPdu.setNumOfVertices(new UnsignedByte(getInt("numberofvertices")));

     //now clear the hashtable ht for the new queries
     ht.clear();

      //set positions
     sqlCoord = sqlCoord + minefield_id + " order by id";
     Vector v = db.getMultiRecords(sqlCoord);
     int i =0;
     for (Enumeration e = v.elements() ; e.hasMoreElements() ;) {
         ht = (Hashtable)e.nextElement();
         if(ht == null) msg("Error !!! Location vector is null !");
         mfPdu.setRefLocationVerticeAt(i,getWorldCoordinate());
         i++;         
     }

     }catch(Exception e) {msg(e.getMessage());}     
 }

 private String get(String name) {
     return ht.get(name).toString();
 }

 private int getInt(String name) {
     int i = 0;
     try {
         i = Integer.parseInt(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return i;
 }
 private double getDouble(String name) {
     double d = 0.0;
     try {
         d = (new Double(get(name))).doubleValue();
     }catch(Exception e) { msg(e.getMessage()); }
     return d;
 }
 private short getShort(String name) {
     short s = 0;
     try {
         s = Short.parseShort(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return s;
 }
 
 public void save(){ 
     String mfid = "";
     sql = makeSQLStr(sql);
     //msg("opening database connection after making sql str");
     //db.open();
     //msg("sql string end product " +sql);
     db.insert(ht,minefield_tablename,sql,1); 
     //msg("Inserted on PDU...");
    
     try {
         mfid = db.getMaxValue(minefield_tablename,"id");
     }catch(Exception e) {
         msg("Error ! while getting MineField Record Id ");
         msg(e.getMessage());
     }
     ht.clear();
     if(wcoords != null && wcoords.length > 0)
         insertWorldCoordinates(mfid);
 }

 private String makeSQLStr(String s) {
     StringBuffer sb = new StringBuffer(s + "(");
     for(int i = 1; i < minefield_cols.length; i++) {
         if(i < minefield_cols.length-1)
	         sb.append(minefield_cols[i].toLowerCase() +",");
         else
			 sb.append(minefield_cols[i].toLowerCase() + ")");
     }
     sb.append(" values ( " + qMarks + " ) ");     
     return sb.toString();
 }
 private void insertWorldCoordinates(String mfid) {
     String sql = "insert into "+location_tablename +
		  "(pdu_id,xcoord, ycoord, zcoord) values(?,?,?,?)";
     WorldCoordinate wcoord = new WorldCoordinate();
     //mfPdu.printValues(2,System.out);
     for(int i=0; i<wcoords.length; i++){
         wcoord = wcoords[i];
         ht.put("pdu_id",mfid);
         ht.put("xcoord",(new Double(wcoord.getX())).toString());
         ht.put("ycoord",(new Double(wcoord.getY())).toString());
         ht.put("zcoord",(new Double(wcoord.getZ())).toString()); 
         try { 
             db.insert(ht,location_tablename,sql,1);
         }catch(Exception e){
             msg("Error while inserting world coordinates");
             e.printStackTrace(System.out);
         }
         ht.clear();
     }
  //   wcoord = null;
 }
 private WorldCoordinate getWorldCoordinate(){
     return (new WorldCoordinate(getDouble("xcoord"), getDouble("ycoord"), getDouble("zcoord")));
 }

 private void msg(String str) {
     System.out.println(str);
 }
 private void msg(int i) {
     System.out.println(i);
 } 
 public MinefieldPdu getPDU() {
     return mfPdu;
 }
}
