/**
 * StartResumePdu database operations (DML's)
 */
package pduDB;

import java.sql.*;
import java.util.*;


import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

import iwt.mine.*;

public class StartResumePduDML {

 private StartResumePdu srPdu;
 private SQLUtil db; //db connection
 private static String startresume_tablename = "dis_startresumepdu_table";
 private static String[] startresume_cols = { //make sure this is in the order
  					    // of the columns in the table
        "id", "disheader_id", "realclocktime", "realtimepast",
    	"simclocktime", "simtimepast","requestid" 
        };
 private static String qMarks = "?,?,?,"+						
          			"?,?,?"
 			  	;
		
 private String sql = "insert into " + startresume_tablename;
 private Hashtable ht;
 private int disHeaderId;

 public StartResumePduDML() {
     System.out.println("StartResumePdu called");
 }
 public StartResumePduDML(StartResumePdu pdu, int disId, SQLUtil dbUtil) {
     srPdu = pdu;
     db = dbUtil;
     disHeaderId = disId;
     fillHash();
 }
 public StartResumePduDML(int disId, SQLUtil dbUtil) {
     srPdu = new StartResumePdu();
     db = dbUtil;
     ht = null;
     disHeaderId = disId;
     fillPdu();
 }

 private void fillHash(){
     ht = new Hashtable();
     ht.put("disheader_id",(new Integer(disHeaderId)).toString());
     
     ClockTime ctime = srPdu.getRealWorldTime();
     ht.put("realclocktime",(new Integer(ctime.getHour())).toString());
     ht.put("realtimepast",(new Integer(ctime.getTimePastHour())).toString());

     ctime = srPdu.getSimulationTime();
     ht.put("simclocktime",(new Integer(ctime.getHour())).toString());
     ht.put("simtimepast",(new Integer(ctime.getTimePastHour())).toString());

     ht.put("requestid",srPdu.getRequestID().toString());
  //   System.out.println("HT in fill hash = "+ ht.toString());
 
 }

 private void fillPdu(){
     String sql = "select * from "+startresume_tablename +
 	          " where disheader_id = " +disHeaderId;
     int startresume_id = -1;
     try{
     ht = db.getRecord(sql);
     if(ht == null) {
         msg("Error !!! No start resume record found for this disheader ");
         return;
     }
     startresume_id = getInt("id");
   
     ClockTime ctime = new ClockTime(getLong("realclocktime"),getLong("realtimepast"));
     srPdu.setRealWorldTime(ctime);

     ctime = new ClockTime(getLong("simclocktime"), getLong("simtimepast"));
     srPdu.setSimulationTime(ctime);

     srPdu.setRequestID(getLong("requestid"));
          
     //now clear the hashtable ht for the new queries
     ht.clear();

     }catch(Exception e) {msg(e.getMessage());}     
 }

 private String get(String name) {
     return ht.get(name).toString();
 }

 private int getInt(String name) {
     int i = 0;
     try {
         i = Integer.parseInt(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return i;
 }
 
 private long getLong(String name) {
     long l = 0;
     try {
         l = Long.parseLong(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return l;
 }

 public void save(){
     sql = makeSQLStr(sql);
     //msg("opening database connection after making sql str");
     //db.open();
//     msg("sql string end product " +sql);
     db.insert(ht,startresume_tablename,sql,1); 
     //msg("Inserted on PDU...");
 }

 private String makeSQLStr(String s) {
     StringBuffer sb = new StringBuffer(s + "(");
     for(int i = 1; i < startresume_cols.length; i++) {
         if(i < startresume_cols.length-1)
	         sb.append(startresume_cols[i].toLowerCase() +",");
         else
			 sb.append(startresume_cols[i].toLowerCase() + ")");
     }
     sb.append(" values ( " + qMarks + " ) ");     
     return sb.toString();
 }

 private void msg(String str) {
     System.out.println(str);
 }
 
 public StartResumePdu getPDU() {
     return srPdu;
 }
}
