/**
 * StopFreezePdu database operations (DML's)
 */
package pduDB;

import java.sql.*;
import java.util.*;


import mil.navy.nps.dis.*;
import mil.navy.nps.util.*;
import mil.navy.nps.disEnumerations.*;

import iwt.mine.*;

public class StopFreezePduDML {

 private StopFreezePdu sfPdu;
 private SQLUtil db; //db connection
 private static String stopfreeze_tablename = "dis_stopfreezepdu_table";
 private static String[] stopfreeze_cols = { //make sure this is in the order
  					    // of the columns in the table
        "id", "disheader_id", "realclocktime", "realtimepast",
    	"requestid","frozenbehavior","reason"
        };
 private static String qMarks = "?,?,?,"+						
          			"?,?,?"
 			  	;
		
 private String sql = "insert into " + stopfreeze_tablename;
 private Hashtable ht;
 private int disHeaderId;

 public StopFreezePduDML() {
     System.out.println("StopFreezePdu called");
 }
 public StopFreezePduDML(StopFreezePdu pdu, int disId, SQLUtil dbUtil) {
     sfPdu = pdu;
     db = dbUtil;
     disHeaderId = disId;
     fillHash();
 }
 public StopFreezePduDML(int disId, SQLUtil dbUtil) {
     sfPdu = new StopFreezePdu();
     db = dbUtil;
     ht = null;
     disHeaderId = disId;
     fillPdu();
 }

 private void fillHash(){
     ht = new Hashtable();
     ht.put("disheader_id",(new Integer(disHeaderId)).toString());
     
     ClockTime ctime = sfPdu.getRealWorldTime();
     ht.put("realclocktime",(new Integer(ctime.getHour())).toString());
     ht.put("realtimepast",(new Integer(ctime.getTimePastHour())).toString());

     ht.put("requestid",sfPdu.getRequestID().toString());
     ht.put("frozenbehavior",sfPdu.getFrozenBehavior().toString());
     ht.put("reason",sfPdu.getReason().toString());
     
  //   System.out.println("HT in fill hash = "+ ht.toString());
 
 }

 private void fillPdu(){
     String sql = "select * from "+stopfreeze_tablename +
 	          " where disheader_id = " +disHeaderId;
     int stopfreeze_id = -1;
     try{
     ht = db.getRecord(sql);
     if(ht == null) {
         msg("Error !!! No stop freeze record found for this disheader ");
         return;
     }
     stopfreeze_id = getInt("id");
   
     ClockTime ctime = new ClockTime(getLong("realclocktime"),getLong("realtimepast"));
     sfPdu.setRealWorldTime(ctime);

     sfPdu.setRequestID(getInt("requestid"));
     sfPdu.setFrozenBehavior(getInt("frozenbehavior"));
     sfPdu.setReason(getInt("reason"));
     //now clear the hashtable ht for the new queries
     ht.clear();

     }catch(Exception e) {msg(e.getMessage());}     
 }

 private String get(String name) {
     return ht.get(name).toString();
 }

 private int getInt(String name) {
     int i = 0;
     try {
         i = Integer.parseInt(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return i;
 }
 private long getLong(String name) {
     long l = 0;
     try {
         l = Long.parseLong(get(name));
     }catch(Exception e) {msg(e.getMessage());}
     return l;
 }
 public void save(){
     sql = makeSQLStr(sql);
     //msg("opening database connection after making sql str");
     //db.open();
    // msg("sql string end product " +sql);
     db.insert(ht,stopfreeze_tablename,sql,1); 
     //msg("Inserted on PDU...");
 }

 private String makeSQLStr(String s) {
     StringBuffer sb = new StringBuffer(s + "(");
     for(int i = 1; i < stopfreeze_cols.length; i++) {
         if(i < stopfreeze_cols.length-1)
	         sb.append(stopfreeze_cols[i].toLowerCase() +",");
         else
			 sb.append(stopfreeze_cols[i].toLowerCase() + ")");
     }
     sb.append(" values ( " + qMarks + " ) ");     
     return sb.toString();
 }

 private void msg(String str) {
     System.out.println(str);
 }
 
 public StopFreezePdu getPDU() {
     return sfPdu;
 }
}
